import java.awt.*;

public class PopupPuzzleCanvas extends PopupPanel {
    PuzzleCanvas X;

    public PopupPuzzleCanvas() {
        super("Puzzle Window", 520,430);
        X=new PuzzleCanvas();
        X.setSize(520,430);
        add(X);
        X.setBackground(Color.black);
        setVisible(true);
        F.setSize(520,430);
    }

    public void resized(int w, int h){
	try {X.setSize(w,h);}
	catch(Exception e) {}
    }
    
}
