import java.awt.*;

public class PopupSelectCanvas extends PopupPanel {
    SelectCanvas S;

    public PopupSelectCanvas() {
        super("Select Window", 660,565);
        S=new SelectCanvas();
        S.setSize(660,565);
        add(S);
        S.setBackground(Color.black);
        setVisible(true);
        F.setSize(660,565);
    }

    public void resized(int w, int h){
	try {S.setSize(w,h);}
	catch(Exception e) {}
    }
    
}
