#! /usr/bin/tclsh
package require Tk

set HEIGHT 1000
canvas .plot -width 1000 -height $HEIGHT 
pack .plot

.plot create rectangle 0 0 1000 1000 -fill black
.plot create rectangle 100 100 500 500 -fill blue -outline white -width 0.1
