import java.awt.*;

public class PopupDeformCanvas extends PopupPanel {
    DeformCanvas Y;

    public PopupDeformCanvas() {
        super("Deform Window", 520,430);
        Y=new DeformCanvas();
        Y.setSize(520,430);
        add(Y);
        Y.setBackground(Color.black);
        setVisible(true);
        F.setSize(520,430);
    }

    public void resized(int w, int h){
	try {Y.setSize(w,h);}
	catch(Exception e) {}
    }
    
}
