import java.awt.*;
import java.math.*;


public class Manager  {
    PictureCanvas P;
    ControlCanvas C;
    SelectCanvas S;
    PuzzleCanvas X;
    DeformCanvas Y;
    public Manager() {}

    public void repaint() {
	try{P.repaint();}
	catch(Exception e) {}

	try{S.repaint();}
	catch(Exception e) {}
	
	try{X.repaint();}
	catch(Exception e) {}

	try{Y.repaint();}
	catch(Exception e) {}

	try{C.repaint();}
	catch(Exception e) {}
    }

}

    

