import java.awt.*;

public class PopupSearchCanvas extends PopupPanel {
    SearchCanvas X;

    public PopupSearchCanvas() {
        super("Search Window", 400,350);
        X=new SearchCanvas();
        X.setSize(400,350);
        add(X);
        X.setBackground(Color.black);
        setVisible(true);
        F.setSize(400,350);
    }

    public void resized(int w, int h){
	try {X.setSize(w,h);}
	catch(Exception e) {}
    }
    
}
